@@ -1,13 +1,14 @@
 # $Header: /home/jesse/DBIx-SearchBuilder/history/SearchBuilder/Handle/Oracle.pm,v 1.14 2002/01/28 06:11:37 jesse Exp $
 
-use strict;
 package DBIx::SearchBuilder::Handle::Oracle;
+
+use strict;
+use warnings;
+
 use base qw/DBIx::SearchBuilder::Handle/;
-use DBD::Oracle qw(:ora_types);
+
+use DBD::Oracle qw(:ora_types ORA_OCI);
          
-use vars qw($VERSION $DBIHandle $DEBUG);
-
-
 =head1 NAME
 
   DBIx::SearchBuilder::Handle::Oracle - An oracle specific Handle object
@@ -42,14 +43,14 @@ sub Connect  {
 	       Host => undef,
 	       @_);
   
-    $self->SUPER::Connect(%args);
+    my $rv = $self->SUPER::Connect(%args);
     
     $self->dbh->{LongTruncOk}=1;
     $self->dbh->{LongReadLen}=8000;
     
     $self->SimpleQuery("ALTER SESSION set NLS_DATE_FORMAT = 'YYYY-MM-DD HH24:MI:SS'");
     
-    return ($DBIHandle); 
+    return ($rv); 
 }
 
 
@@ -238,18 +239,29 @@ sub ApplyLimits {
 
 takes an incomplete SQL SELECT statement and massages it to return a DISTINCT result set.
 
-
 =cut
 
 sub DistinctQuery {
     my $self = shift;
     my $statementref = shift;
-    my $table = shift;
+    my $sb = shift;
+    my $table = $sb->Table;
 
-    # Wrapper select query in a subselect as Oracle doesn't allow
+    # Wrapp select query in a subselect as Oracle doesn't allow
     # DISTINCT against CLOB/BLOB column types.
-    $$statementref = "SELECT main.* FROM ( SELECT DISTINCT main.id FROM $$statementref ) distinctquery, $table main WHERE (main.id = distinctquery.id) ";
-
+    if ($sb->_OrderClause =~ /(?<!main)\./) {
+        # If we are ordering by something not in 'main', we need to GROUP
+        # BY and adjust the ORDER_BY accordingly
+        local $sb->{group_by} = [@{$sb->{group_by} || []}, {FIELD => 'id'}];
+        local $sb->{order_by} = [map {($_->{ALIAS} and $_->{ALIAS} ne "main") ? {%{$_}, FIELD => "min(".$_->{FIELD}.")"}: $_} @{$sb->{order_by}}];
+        my $group = $sb->_GroupClause;
+        my $order = $sb->_OrderClause;
+        $$statementref = "SELECT main.* FROM ( SELECT main.id FROM $$statementref $group $order ) distinctquery, $table main WHERE (main.id = distinctquery.id)";
+    } else {
+        $$statementref = "SELECT main.* FROM ( SELECT DISTINCT main.id FROM $$statementref ) distinctquery, $table main WHERE (main.id = distinctquery.id) ";
+        $$statementref .= $sb->_GroupClause;
+        $$statementref .= $sb->_OrderClause;
+    }
 }
 
 
@@ -267,6 +279,15 @@ sub BinarySafeBLOBs {
     return(undef);
 }
 
+=head2 DatabaseVersion
+
+Returns value of ORA_OCI constant, see L<DBI/Constants>.
+
+=cut
+
+sub DatabaseVersion {
+    return ''. ORA_OCI;
+}
 
 1;
 
