@@ -106,8 +106,10 @@ sub Next {
   my $self=shift;
 
   return undef unless defined  $self->{data}[ $self->{curp} ];
-
+    
   my $cur =  $self->{data}[ $self->{curp} ];
+  # do the search to avoid the count query and the search
+  $cur->_DoSearch if $cur->{'must_redo_search'};
   if ( $cur->_ItemsCounter == $cur->Count ) {
 	# move to the next element
 	$self->{curp}++;
